/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;

    
    /// <summary> Defines a many-valued association with many-to-many multiplicity. 
    /// <para>
    ///  Every many-to-many association has two sides, the owning side
    /// and the non-owning, or inverse, side.  The join table is specified
    /// on the owning side. If the association is bidirectional, either
    /// side may be designated as the owning side.  If the relationship is
    /// bidirectional, the non-owning side must use the <c>mappedBy</c> element of
    /// the <c>ManyToMany</c> annotation to specify the relationship field or
    /// property of the owning side.
    /// </para>
    /// <para>
    ///  The join table for the relationship, if not defaulted, is
    /// specified on the owning side.
    /// </para>
    /// <para>
    ///  The <c>ManyToMany</c> annotation may be used within an
    /// embeddable class contained within an entity class to specify a
    /// relationship to a collection of entities. If the relationship is
    /// bidirectional and the entity containing the embeddable class is the
    /// owner of the relationship, the non-owning side must use the
    /// <c>mappedBy</c> element of the <c>ManyToMany</c>
    /// annotation to specify the relationship field or property of the
    /// embeddable class. The dot (".") notation syntax must be used in the
    /// <c>mappedBy</c> element to indicate the relationship
    /// attribute within the embedded attribute.  The value of each
    /// identifier used with the dot notation is the name of the respective
    /// embedded field or property.
    /// </para>
    /// <example><code>
    /// 
    ///    Example 1:
    /// 
    ///    // In Customer class:
    /// 
    ///    &#91;ManyToMany&#93; 
    ///    &#91;JoinTable(name="CUST_PHONES")&#93;
    ///    public Set&#060;PhoneNumber&#062; getPhones() { return phones; }
    /// 
    ///    // In PhoneNumber class:
    /// 
    ///    &#91;ManyToMany(mappedBy="phones")&#93;
    ///    public Set&#060;Customer&#062; getCustomers() { return customers; }
    /// 
    ///    Example 2:
    /// 
    ///    // In Customer class:
    /// 
    ///    &#91;ManyToMany(targetEntity=com.acme.PhoneNumber.class)&#93;
    ///    public Set getPhones() { return phones; }
    /// 
    ///    // In PhoneNumber class:
    /// 
    ///    &#91;ManyToMany(targetEntity=com.acme.Customer.class, mappedBy="phones")&#93;
    ///    public Set getCustomers() { return customers; }
    /// 
    ///    Example 3:
    /// 
    ///    // In Customer class:
    /// 
    ///    &#91;ManyToMany&#93;
    ///    &#91;JoinTable(name="CUST_PHONE")&#93;
    ///            &#91;JoinColumn(AttributeRef="CUST_PHONE",name="CUST_ID", referencedColumnName="ID")&#93;
    ///            &#91;JoinColumn(AttributeRef="CUST_PHONE",name="PHONE_ID", referencedColumnName="ID", Inverse=true)&#93;
    ///        )
    ///    public Set&#060;PhoneNumber&#062; getPhones() { return phones; }
    /// 
    ///    // In PhoneNumberClass:
    /// 
    ///    &#91;ManyToMany(mappedBy="phones")&#93;
    ///    public Set&#060;Customer&#062; getCustomers() { return customers; }
    /// </code></example>
    /// 
    /// <seealso cref="JoinTable"/>
    /// 
    /// 
    /// </summary>
    [System.AttributeUsage(System.AttributeTargets.Field | System.AttributeTargets.Property)] 

public sealed class ManyToMany : System.Attribute {

    /// <summary> (Optional) The entity class that is the target of the
    /// association. Optional only if the collection-valued
    /// relationship property is defined using generics. Must be
    /// specified otherwise.
    /// <para>
    ///  Defaults to the parameterized type of
    /// the collection when defined using generics.
    /// </para>
    /// </summary>
    public System.Type TargetEntity = typeof(void);

    /// <summary> 
    /// (Optional) The operations that must be cascaded to the target
    /// of the association.  
    /// 
    ///  When the target collection is a <see cref="T:System.Collections.Generic.Dictionary`2"/>, the <c>cascade</c> element applies to the
    /// map value.
    /// 
    ///  Defaults to no operations being cascaded.
    /// </summary>
    public CascadeType[] Cascade = {};

    /// <summary> (Optional) Whether the association should be lazily loaded or
    /// must be eagerly fetched. The EAGER strategy is a requirement on
    /// the persistence provider runtime that the associated entities
    /// must be eagerly fetched.  The LAZY strategy is a hint to the
    /// persistence provider runtime.
    /// </summary>
    public FetchType Fetch = FetchType.LAZY;

    /// <summary> 
    /// The field that owns the relationship. Required unless 
    /// the relationship is unidirectional.
    /// </summary>
    public string MappedBy = "";
}
}