/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;

    /// <summary> Defines a single-valued association to another entity that has
    /// one-to-one multiplicity. 
    /// <para>
    /// It is not normally necessary to specify
    /// the associated target entity explicitly since it can usually be
    /// inferred from the type of the object being referenced. 
    /// </para><para>If the relationship is
    /// bidirectional, the non-owning side must use the <c>mappedBy</c> element of
    /// the <c>OneToOne</c> annotation to specify the relationship field or
    /// property of the owning side.
    /// </para><para>
    ///  The <c>OneToOne</c> annotation may be used within an
    /// embeddable class to specify a relationship from the embeddable
    /// class to an entity class.
    /// </para><para>
    /// If the relationship is bidirectional and
    /// the entity containing the embeddable class is on the owning side of
    /// the relationship, the non-owning side must use the
    /// <c>mappedBy</c> element of the <c>OneToOne</c>
    /// annotation to specify the relationship field or property of the
    /// embeddable class. 
    /// </para><para>
    /// The dot (".") notation syntax must be used in the
    /// <c>mappedBy</c> element to indicate the relationship attribute within the
    /// embedded attribute.  The value of each identifier used with the dot
    /// notation is the name of the respective embedded field or property.
    /// </para>
    /// <example><code>
    ///    Example 1: One-to-one association that maps a foreign key column
    /// 
    ///    // On Customer class:
    /// 
    ///    &#91;OneToOne(optional=false)&#93;
    ///    &#91;JoinColumn(
    ///    	name="CUSTREC_ID", unique=true, nullable=false, updatable=false)&#93;
    ///    public CustomerRecord getCustomerRecord() { return customerRecord; }
    /// 
    ///    // On CustomerRecord class:
    /// 
    ///    &#91;OneToOne(optional=false, mappedBy="customerRecord")&#93;
    ///    public Customer getCustomer() { return customer; }
    /// 
    /// 
    ///    Example 2: One-to-one association that assumes both the source and target share the same primary key values. 
    /// 
    ///    // On Employee class:
    /// 
    ///    &#91;Entity&#93;
    ///    public class Employee {
    ///    	&#91;Id&#93; Integer id;
    ///    
    ///    	&#91;OneToOne&#93;MapsId
    ///    	EmployeeInfo info;
    ///    	...
    ///    }
    /// 
    ///    // On EmployeeInfo class:
    /// 
    ///    &#91;Entity&#93;
    ///    public class EmployeeInfo {
    ///    	&#91;Id&#93; Integer id;
    ///    	...
    ///    }
    /// 
    /// 
    ///    Example 3: One-to-one association from an embeddable class to another entity.
    /// 
    ///    &#91;Entity&#93;
    ///    public class Employee {
    ///       &#91;Id&#93; int id;
    ///       &#91;Embedded&#93; LocationDetails location;
    ///       ...
    ///    }
    /// 
    ///    &#91;Embeddable&#93;
    ///    public class LocationDetails {
    ///       int officeNumber;
    ///       &#93;OneToOne &#93;ParkingSpot parkingSpot;
    ///       ...
    ///    }
    /// 
    ///    &#91;Entity&#93;
    ///    public class ParkingSpot {
    ///       &#91;Id&#93; int id;
    ///       String garage;
    ///       &#91;OneToOne(mappedBy="location.parkingSpot")&#93; Employee assignedTo;
    ///        ... 
    ///    } 
    /// 
    /// </code></example>
    /// 
    /// 
    /// </summary>
    [System.AttributeUsage(System.AttributeTargets.Field | System.AttributeTargets.Property)] 


public sealed class OneToOne : System.Attribute {

    /// <summary> 
    /// (Optional) The entity class that is the target of 
    /// the association. 
    /// 
    ///  Defaults to the type of the field or property 
    /// that stores the association. 
    /// </summary>
    public System.Type TargetEntity = typeof(void);

    /// <summary> (Optional) The operations that must be cascaded to 
    /// the target of the association.
    /// 
    ///  By default no operations are cascaded.
    /// </summary>
    public CascadeType[] Cascade = {};

    /// <summary> 
    /// (Optional) Whether the association should be lazily 
    /// loaded or must be eagerly fetched. The EAGER 
    /// strategy is a requirement on the persistence provider runtime that 
    /// the associated entity must be eagerly fetched. The LAZY 
    /// strategy is a hint to the persistence provider runtime.
    /// </summary>
    public FetchType Fetch = FetchType.EAGER;

    /// <summary> 
    /// (Optional) Whether the association is optional. If set 
    /// to false then a non-null relationship must always exist.
    /// </summary>
    public bool Optional = true;

    /// <summary> (Optional) The field that owns the relationship. This 
     /// element is only specified on the inverse (non-owning) 
     /// side of the association.
    /// </summary>
    public string MappedBy = "";


    /// <summary> (Optional) Whether to apply the remove operation to entities that have
    /// been removed from the relationship and to cascade the remove operation to
    /// those entities.
    /// 
    /// </summary>
    public bool OrphanRemoval = false;
}
}