/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems, Oracle Corporation. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
///     Oracle committers - EclipseLink implementation of Persistence and PersistenceUtil 
/// 
/*****************************************************************************/
namespace NPersistence {
    using NPersistence.spi;
    using System;
    using System.Reflection;
    using System.Collections;
    using System.Collections.Generic;



    /// <summary> Bootstrap class that is used to obtain an <see cref="EntityManagerFactory"/>
    /// in .NET environments.  
    /// <para>
    ///  The <c>Persistence</c> class is also used to obtain a 
    /// <see cref="PersistenceUtil"/> instance.
    /// </para>
    /// 
    /// </summary>
    public class Persistence
    {

        /// <summary>
        /// Create and return an EntityManagerFactory for the named
        /// persistence unit.
        /// 
        /// </summary>
        /// <param name='persistenceUnitName'>the name of the persistence unit</param>
        ///            
        /// <returns> the factory that creates EntityManagers configured according to
        ///         the specified persistence unit</returns>
        public static EntityManagerFactory CreateEntityManagerFactory(string persistenceUnitName)
        {
            return CreateEntityManagerFactory(persistenceUnitName, null);
        }

        /// <summary>
        /// Create and return an EntityManagerFactory for the named persistence unit
        /// using the given properties.
        /// </summary>
        /// <param name='persistenceUnitName'>the name of the persistence unit</param>
        /// <param name='properties'>
        ///            Additional properties to use when creating the factory. The
        ///            values of these properties override any values that may have
        ///            been configured elsewhere.</param>
        /// <returns> the factory that creates EntityManagers configured according to
        ///         the specified persistence unit.</returns>
        /// 
        public static EntityManagerFactory CreateEntityManagerFactory(string persistenceUnitName, Dictionary<object, object> properties)
        {

            EntityManagerFactory emf = null;
            PersistenceProviderResolver resolver = PersistenceProviderResolverHolder.GetPersistenceProviderResolver();

            IList<PersistenceProvider> providers = resolver.GetPersistenceProviders();

            foreach (PersistenceProvider provider in providers)
            {
                emf = provider.CreateEntityManagerFactory(persistenceUnitName, properties);
                if (emf != null)
                {
                    break;
                }
            }
            if (emf == null)
            {
                throw new PersistenceException("No Persistence provider for EntityManager named " + persistenceUnitName);
            }
            return emf;
        }

        /// <summary>
        /// Return the PersistenceUtil instance</summary>
        /// <returns> PersistenceUtil instance</returns>
        /// 
        /// 
        public static PersistenceUtil GetPersistenceUtil()
        {
            return new PersistenceUtilImpl();
        }


        /// <summary>
        /// Implementation of PersistenceUtil interface
        /// 
        /// </summary>
        private class PersistenceUtilImpl : PersistenceUtil
        {
            public bool IsLoaded(object entity, string attributeName)
            {
                PersistenceProviderResolver resolver = PersistenceProviderResolverHolder.GetPersistenceProviderResolver();

                IList<PersistenceProvider> providers = resolver.GetPersistenceProviders();

                foreach (PersistenceProvider provider in providers) {
                    LoadState loadstate = provider.GetProviderUtil().IsLoadedWithoutReference(entity, attributeName);
                    if(loadstate == LoadState.LOADED) 
                    {
                        return true;
                    } 
                    else if (loadstate == LoadState.NOT_LOADED) 
                    {
                        return false;
                    } // else continue
                }

                //None of the providers could determine the load state try isLoadedWithReference
                foreach (PersistenceProvider provider in providers) {
                    LoadState loadstate = provider.GetProviderUtil().IsLoadedWithReference(entity, attributeName);
                    if(loadstate == LoadState.LOADED) 
                    {
                        return true;
                    } 
                    else if (loadstate == LoadState.NOT_LOADED) 
                    {
                        return false;
                    } // else continue
                }

                //None of the providers could determine the load state.
                return true;
            }

            public bool IsLoaded(object entity)
            {
                PersistenceProviderResolver resolver = PersistenceProviderResolverHolder.GetPersistenceProviderResolver();

                IList<PersistenceProvider> providers = resolver.GetPersistenceProviders();

                foreach (PersistenceProvider provider in providers) {
                    LoadState loadstate = provider.GetProviderUtil().IsLoaded(entity);
                    if(loadstate == LoadState.LOADED) {
                        return true;
                    } else if (loadstate == LoadState.NOT_LOADED) {
                        return false;
                    } // else continue
                }
                //None of the providers could determine the load state
                return true;
            }
        }

        /// <summary>
        /// This final String is deprecated and should be removed and is only here for TCK backward compatibility
        /// 
        /// @deprecated
        /// </summary>
        public const string PERSISTENCE_PROVIDER = "NPersistence.spi.PeristenceProvider";

        /// <summary>
        /// This instance variable is deprecated and should be removed and is only here for TCK backward compatibility
        /// 
        /// @deprecated
        /// </summary>
        //internal
        //static readonly Set<PersistenceProvider> providers = new HashSet<PersistenceProvider>();
    }
}