/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;


    /// <summary> Defines a primary key generator that may be 
    /// referenced by name when a generator element is specified for 
    /// the <see cref="GeneratedValue"/> annotation.
    /// <para>
    /// A table generator 
    /// may be specified on the entity class or on the primary key 
    /// field or property. 
    /// </para><para>
    /// The scope of the generator name is global 
    /// to the persistence unit (across all generator types).
    /// </para>
    /// <example><code>
    ///    Example 1:
    ///    
    ///    &#91;Entity&#93; public class Employee {
    ///        ...
    ///        &#91;TableGenerator(
    ///            name="empGen", 
    ///            table="ID_GEN", 
    ///            pkColumnName="GEN_KEY", 
    ///            valueColumnName="GEN_VALUE", 
    ///            pkColumnValue="EMP_ID", 
    ///            allocationSize=1)&#93;
    ///        &#91;Id&#93;
    ///        &#91;GeneratedValue(strategy=TABLE, generator="empGen")&#93;
    ///        int id;
    ///        ...
    ///    }
    ///    
    ///    Example 2:
    ///    
    ///    &#91;Entity&#93; public class Address {
    ///        ...
    ///        &#91;TableGenerator(
    ///            name="addressGen", 
    ///            table="ID_GEN", 
    ///            pkColumnName="GEN_KEY", 
    ///            valueColumnName="GEN_VALUE", 
    ///            pkColumnValue="ADDR_ID")&#93;&#93;
    ///        &#91;Id&#93;
    ///        &#91;GeneratedValue(strategy=TABLE, generator="addressGen")&#93;
    ///        int id;
    ///        ...
    ///    }
    /// </code></example>
    /// 
    /// <seealso cref="GeneratedValue"/>
    /// 
    /// 
    /// </summary>
    [System.AttributeUsage(System.AttributeTargets.Class | System.AttributeTargets.Field | System.AttributeTargets.Property)] 

public sealed class TableGenerator : System.Attribute {

    /// <summary> 
    /// (Required) A unique generator name that can be referenced 
    /// by one or more classes to be the generator for id values
    /// </summary>
    public string Name;

    /// <summary> 
    /// (Optional) Name of table that stores the generated id values. 
    ///  Defaults to a name chosen by persistence provider
    ///  </summary>
    public string Table = "";

    /// <summary> (Optional) The catalog of the table. 
    ///  Defaults to the default catalog
    ///  </summary>
    public string Catalog = "";

    /// <summary> (Optional) The schema of the table. 
    ///  Defaults to the default schema for user
    ///  </summary>
    public string Schema = "";

    /// <summary> 
    /// (Optional) Name of the primary key column in the table.
    ///  Defaults to a provider-chosen name
    ///  </summary>
    public string PkColumnName = "";

    /// <summary> 
    /// (Optional) Name of the column that stores the last value generated.
    ///  Defaults to a provider-chosen name
    ///  </summary>
    public string ValueColumnName = "";

    /// <summary> (Optional) The primary key value in the generator table 
    /// that distinguishes this set of generated values from others 
    /// that may be stored in the table.
    ///  Defaults to a provider-chosen value to store in the 
    /// primary key column of the generator table
    /// </summary>
    public string PkColumnValue = "";

    /// <summary> 
    /// (Optional) The initial value to be used to initialize the column
    /// that stores the last value generated
    /// </summary>
    public int InitialValue = 0;

    /// <summary> (Optional) The amount to increment by when allocating id 
    /// numbers from the generator
    /// </summary>
    public int AllocationSize = 50;
}
}